"""
ByteDocs Django - Core Type Definitions
Based on ByteDocs Express implementation
"""

from typing import Any, Dict, List, Optional, Literal, Callable
from dataclasses import dataclass, field
from enum import Enum


class ParameterLocation(str, Enum):
    """Parameter location types"""
    PATH = "path"
    QUERY = "query"
    HEADER = "header"
    COOKIE = "cookie"


@dataclass
class BaseURLOption:
    """Base URL configuration for multiple environments"""
    name: str
    url: str


@dataclass
class AuthConfig:
    """Authentication configuration"""
    enabled: bool = False
    type: Optional[Literal["basic", "api_key", "bearer", "session"]] = None
    username: Optional[str] = None
    password: Optional[str] = None
    api_key: Optional[str] = None
    api_key_header: Optional[str] = None
    realm: Optional[str] = None  # For basic auth

    # Session-based auth
    session_expire: int = 60  # Minutes
    ip_ban_enabled: bool = False
    ip_ban_max_attempts: int = 5
    ip_ban_duration: int = 30  # Minutes
    admin_whitelist_ips: List[str] = field(default_factory=list)


@dataclass
class UIConfig:
    """UI configuration"""
    theme: Literal["auto", "green", "blue", "purple", "red", "orange", "teal", "pink"] = "green"
    dark_mode: bool = False
    show_try_it: bool = True
    show_schemas: bool = True


@dataclass
class AIFeatures:
    """AI features configuration"""
    chat_enabled: bool = True
    doc_generation_enabled: bool = True
    model: Optional[str] = None
    max_tokens: Optional[int] = None
    max_completion_tokens: Optional[int] = None
    temperature: float = 0.7


@dataclass
class AIConfig:
    """AI configuration"""
    provider: Literal["openai", "gemini", "anthropic", "openrouter"] = "openai"
    api_key: Optional[str] = None
    enabled: bool = False
    features: AIFeatures = field(default_factory=AIFeatures)
    settings: Dict[str, Any] = field(default_factory=dict)


def _default_exclude_paths():
    """Default paths to exclude from documentation"""
    return ["/admin", "/static", "/__debug__"]


@dataclass
class ByteDocsConfig:
    """Main ByteDocs configuration"""
    title: str = "API Documentation"
    version: str = "1.0.0"
    description: Optional[str] = None
    base_url: Optional[str] = None  # Backward compatibility
    base_urls: List[BaseURLOption] = field(default_factory=list)
    docs_path: str = "/docs"
    auto_detect: bool = True
    exclude_paths: List[str] = field(default_factory=_default_exclude_paths)
    auth_config: Optional[AuthConfig] = None
    ui_config: UIConfig = field(default_factory=UIConfig)
    ai_config: Optional[AIConfig] = None


@dataclass
class Schema:
    """OpenAPI schema definition"""
    type: str
    format: Optional[str] = None
    description: Optional[str] = None
    properties: Optional[Dict[str, "Schema"]] = None
    items: Optional["Schema"] = None
    required: Optional[List[str]] = None
    example: Any = None
    enum: Optional[List[Any]] = None
    default: Any = None
    nullable: bool = False
    minimum: Optional[float] = None
    maximum: Optional[float] = None
    min_length: Optional[int] = None
    max_length: Optional[int] = None
    pattern: Optional[str] = None
    ref: Optional[str] = None


@dataclass
class Parameter:
    """OpenAPI parameter definition"""
    name: str
    location: ParameterLocation  # 'in' is reserved keyword in Python
    schema: Schema
    description: Optional[str] = None
    required: bool = False
    example: Any = None


@dataclass
class RequestBody:
    """OpenAPI request body definition"""
    content: Dict[str, Dict[str, Any]]
    description: Optional[str] = None
    required: bool = True


@dataclass
class ResponseDef:
    """OpenAPI response definition"""
    description: str
    content: Optional[Dict[str, Dict[str, Any]]] = None
    headers: Optional[Dict[str, Dict[str, Any]]] = None


@dataclass
class Endpoint:
    """Endpoint definition"""
    path: str
    method: str
    summary: Optional[str] = None
    description: Optional[str] = None
    tags: List[str] = field(default_factory=list)
    parameters: List[Parameter] = field(default_factory=list)
    request_body: Optional[RequestBody] = None
    responses: Dict[str, ResponseDef] = field(default_factory=dict)
    security: Optional[List[Dict[str, List[str]]]] = None
    operation_id: Optional[str] = None
    deprecated: bool = False


@dataclass
class RouteInfo:
    """Route information extracted from Django"""
    method: str
    path: str
    handler: Callable
    summary: Optional[str] = None
    description: Optional[str] = None
    parameters: List[Parameter] = field(default_factory=list)
    request_body: Optional[RequestBody] = None
    responses: Dict[str, ResponseDef] = field(default_factory=dict)
    tags: List[str] = field(default_factory=list)


@dataclass
class Section:
    """Section for grouping endpoints"""
    name: str
    endpoints: List[Endpoint]


@dataclass
class OpenAPIServer:
    """OpenAPI server definition"""
    url: str
    description: Optional[str] = None


@dataclass
class OpenAPIComponents:
    """OpenAPI components"""
    schemas: Optional[Dict[str, Schema]] = None
    responses: Optional[Dict[str, ResponseDef]] = None
    parameters: Optional[Dict[str, Parameter]] = None
    request_bodies: Optional[Dict[str, RequestBody]] = None
    security_schemes: Optional[Dict[str, Any]] = None


@dataclass
class OpenAPISpec:
    """OpenAPI specification"""
    openapi: str = "3.0.3"
    info: Dict[str, Any] = field(default_factory=dict)
    servers: List[OpenAPIServer] = field(default_factory=list)
    paths: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    components: Optional[OpenAPIComponents] = None
    security: Optional[List[Dict[str, List[str]]]] = None
    tags: Optional[List[Dict[str, Any]]] = None


@dataclass
class SessionData:
    """Session data for authentication"""
    id: str
    created_at: float
    expires_at: float
    ip: str


@dataclass
class IPBanRecord:
    """IP ban record for authentication"""
    ip: str
    attempts: int
    banned_at: Optional[float] = None
    expires_at: Optional[float] = None



"""
ByteDocs Django - AST Analyzer
Analyze Python source code using AST to extract response schemas and request bodies
Based on ByteDocs FastAPI AST analyzer
"""

import ast
import inspect
from typing import Any, Dict, Optional, Callable, List


def analyze_handler_source(handler: Callable, source_code: Optional[str] = None) -> Dict[str, Any]:
    """
    Analyze handler function source code using AST
    Returns dict with 'responses' and 'request_body_fields'
    """
    result = {
        'responses': {},
        'request_body_fields': []
    }

    if source_code is None:
        try:
            source_code = inspect.getsource(handler)
        except (OSError, TypeError):
            return result

    try:
        tree = ast.parse(source_code)

        # Find the function definition (both sync and async)
        for node in ast.walk(tree):
            if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
                if node.name == handler.__name__:
                    # Analyze return statements
                    responses = analyze_return_statements(node)
                    if responses:
                        result['responses'] = responses

                    # Analyze request body usage
                    body_fields = analyze_request_body_usage(node)
                    if body_fields:
                        result['request_body_fields'] = body_fields
                    break

    except SyntaxError as e:
        pass  # Can't parse, skip AST analysis

    return result


def analyze_return_statements(func_node) -> Dict[str, Any]:
    """
    Analyze return statements to infer response structure
    Returns dict of status_code -> response_schema
    Accepts both ast.FunctionDef and ast.AsyncFunctionDef
    """
    responses = {}

    for node in ast.walk(func_node):
        # Look for return statements
        if isinstance(node, ast.Return) and node.value:
            schema = infer_schema_from_ast_node(node.value)
            if schema:
                # Default to 200 status
                responses['200'] = {
                    'description': 'Successful operation',
                    'content': {
                        'application/json': {
                            'schema': schema
                        }
                    }
                }

        # Look for Django JsonResponse/Response with status parameter
        if isinstance(node, ast.Call):
            if hasattr(node.func, 'attr'):
                if node.func.attr in ['JsonResponse', 'Response']:
                    # Try to find status argument
                    status_code = '200'
                    content_value = None

                    for keyword in node.keywords:
                        if keyword.arg == 'status':
                            if isinstance(keyword.value, ast.Constant):
                                status_code = str(keyword.value.value)
                        elif keyword.arg in ['data', 'content']:
                            content_value = keyword.value

                    # If we found content, try to infer schema
                    if content_value:
                        schema = infer_schema_from_ast_node(content_value)
                        if schema:
                            responses[status_code] = {
                                'description': get_status_description(int(status_code)),
                                'content': {
                                    'application/json': {
                                        'schema': schema
                                    }
                                }
                            }

    return responses


def analyze_request_body_usage(func_node) -> List[str]:
    """
    Analyze how request body parameter is used to infer required fields
    Returns list of field names that are accessed
    Accepts both ast.FunctionDef and ast.AsyncFunctionDef
    """
    fields = []

    # Get parameter names
    param_names = [arg.arg for arg in func_node.args.args]

    # Look for attribute access on parameters (e.g., request.data, serializer.data)
    for node in ast.walk(func_node):
        if isinstance(node, ast.Attribute):
            if isinstance(node.value, ast.Name):
                if node.value.id in param_names:
                    fields.append(node.attr)

    return list(set(fields))  # Remove duplicates


def infer_schema_from_ast_node(node: ast.AST) -> Optional[Dict[str, Any]]:
    """
    Infer OpenAPI schema from AST node
    """
    # Dict literal: {"key": "value", "num": 123}
    if isinstance(node, ast.Dict):
        properties = {}
        for key_node, value_node in zip(node.keys, node.values):
            if isinstance(key_node, ast.Constant):
                key = str(key_node.value)
                value_schema = infer_schema_from_ast_node(value_node)
                if value_schema:
                    properties[key] = value_schema

        if properties:
            return {
                'type': 'object',
                'properties': properties
            }

    # List literal: [...]
    elif isinstance(node, ast.List):
        if node.elts:
            # Infer from first element
            item_schema = infer_schema_from_ast_node(node.elts[0])
            return {
                'type': 'array',
                'items': item_schema or {'type': 'object'}
            }
        return {
            'type': 'array',
            'items': {'type': 'object'}
        }

    # Constant/Literal values
    elif isinstance(node, ast.Constant):
        value = node.value
        if isinstance(value, str):
            return {'type': 'string', 'example': value}
        elif isinstance(value, bool):
            return {'type': 'boolean', 'example': value}
        elif isinstance(value, int):
            return {'type': 'integer', 'example': value}
        elif isinstance(value, float):
            return {'type': 'number', 'example': value}

    # String literal (older Python)
    elif isinstance(node, ast.Str):
        return {'type': 'string', 'example': node.s}

    # Number literal (older Python)
    elif isinstance(node, ast.Num):
        if isinstance(node.n, int):
            return {'type': 'integer', 'example': node.n}
        else:
            return {'type': 'number', 'example': node.n}

    # Variable reference (can't infer type)
    elif isinstance(node, ast.Name):
        return {'type': 'object'}

    # Call expression (function call result)
    elif isinstance(node, ast.Call):
        # Check for dict() call
        if isinstance(node.func, ast.Name) and node.func.id == 'dict':
            properties = {}
            for keyword in node.keywords:
                if keyword.arg:
                    value_schema = infer_schema_from_ast_node(keyword.value)
                    if value_schema:
                        properties[keyword.arg] = value_schema

            if properties:
                return {
                    'type': 'object',
                    'properties': properties
                }

        # Check for list() call
        elif isinstance(node.func, ast.Name) and node.func.id == 'list':
            return {
                'type': 'array',
                'items': {'type': 'object'}
            }

    return None


def get_status_description(status: int) -> str:
    """Get HTTP status description"""
    descriptions = {
        200: 'Successful operation',
        201: 'Resource created successfully',
        204: 'No content',
        400: 'Bad request',
        401: 'Unauthorized',
        403: 'Forbidden',
        404: 'Resource not found',
        422: 'Validation error',
        500: 'Internal server error',
    }
    return descriptions.get(status, f'HTTP {status}')


def enhance_schema_with_ast(
    existing_schema: Optional[Dict[str, Any]],
    handler: Callable,
    source_code: Optional[str] = None
) -> Dict[str, Any]:
    """
    Enhance existing schema with AST analysis
    Combines DRF serializer schema with AST-detected response examples
    """
    if existing_schema is None:
        existing_schema = {}

    ast_result = analyze_handler_source(handler, source_code)

    # If we found responses from AST, use them
    if ast_result['responses']:
        # Merge with existing schema
        for status, response in ast_result['responses'].items():
            if status not in existing_schema:
                existing_schema[status] = response
            else:
                # Merge schemas if both exist
                if 'content' in response and 'application/json' in response['content']:
                    ast_schema = response['content']['application/json'].get('schema', {})
                    existing_content = existing_schema[status].get('content', {})
                    existing_json = existing_content.get('application/json', {})
                    existing_json_schema = existing_json.get('schema', {})

                    # If existing schema is generic, replace with AST schema
                    if existing_json_schema.get('type') == 'object' and not existing_json_schema.get('properties'):
                        if ast_schema.get('properties'):
                            existing_json['schema'] = ast_schema
                            existing_content['application/json'] = existing_json
                            existing_schema[status]['content'] = existing_content

    return existing_schema


def get_handler_source_code(handler: Callable) -> Optional[str]:
    """
    Get source code of handler function
    """
    try:
        return inspect.getsource(handler)
    except (OSError, TypeError):
        return None



"""
AI Types and Configuration
Based on bytedocs-go AI implementation
"""

from dataclasses import dataclass, field
from typing import Dict, Any, Optional, Protocol
from abc import ABC, abstractmethod


@dataclass
class AIFeatures:
    """AI feature configuration"""
    chat_enabled: bool = True
    doc_generation_enabled: bool = False
    model: str = ""
    max_tokens: int = 0
    max_completion_tokens: int = 0
    temperature: float = 0.7


@dataclass
class AIConfig:
    """AI configuration for providers"""
    provider: str  # openai, gemini, anthropic, openrouter, etc.
    api_key: str = ""
    enabled: bool = False
    features: AIFeatures = field(default_factory=AIFeatures)
    settings: Dict[str, Any] = field(default_factory=dict)


@dataclass
class ChatRequest:
    """Chat request model"""
    message: str
    context: Optional[str] = None
    endpoint: Optional[Any] = None
    metadata: Optional[Dict[str, Any]] = None


@dataclass
class ChatResponse:
    """Chat response model"""
    response: str
    provider: str
    model: Optional[str] = None
    tokens_used: Optional[int] = None
    error: Optional[str] = None


class AIClient(ABC):
    """Base AI client interface"""

    @abstractmethod
    async def chat(self, request: ChatRequest) -> ChatResponse:
        """Send chat request and get response"""
        pass

    @abstractmethod
    def get_provider(self) -> str:
        """Get provider name"""
        pass

    @abstractmethod
    def get_model(self) -> str:
        """Get model name"""
        pass


class ClientFactory(Protocol):
    """Protocol for client factory functions"""

    def __call__(self, config: AIConfig) -> AIClient:
        """Create AI client from config"""
        ...



"""
ByteDocs Django - Views
View functions for documentation endpoints
"""

from django.http import HttpRequest, HttpResponse
from .core.bytedocs import get_bytedocs_instance
from .ui.handlers import (
    create_docs_ui_handler,
    create_api_data_handler,
    create_openapi_json_handler,
    create_openapi_yaml_handler,
    create_chat_handler,
)


def docs_ui(request: HttpRequest) -> HttpResponse:
    """Serve documentation UI"""
    bytedocs = get_bytedocs_instance()
    if bytedocs is None:
        return HttpResponse(
            "ByteDocs not initialized. Please call setup_bytedocs() in your settings or urls.",
            status=500
        )

    # Lazy detect routes on first access
    if not bytedocs._detected:
        bytedocs.detect_routes()

    handler = create_docs_ui_handler(bytedocs)
    return handler(request)


def api_data(request: HttpRequest) -> HttpResponse:
    """Serve API data as JSON"""
    bytedocs = get_bytedocs_instance()
    if bytedocs is None:
        return HttpResponse(
            "ByteDocs not initialized. Please call setup_bytedocs() in your settings or urls.",
            status=500
        )

    # Lazy detect routes on first access
    if not bytedocs._detected:
        bytedocs.detect_routes()

    handler = create_api_data_handler(bytedocs)
    return handler(request)


def openapi_json(request: HttpRequest) -> HttpResponse:
    """Serve OpenAPI specification as JSON"""
    bytedocs = get_bytedocs_instance()
    if bytedocs is None:
        return HttpResponse(
            "ByteDocs not initialized. Please call setup_bytedocs() in your settings or urls.",
            status=500
        )

    handler = create_openapi_json_handler(bytedocs)
    return handler(request)


def openapi_yaml(request: HttpRequest) -> HttpResponse:
    """Serve OpenAPI specification as YAML"""
    bytedocs = get_bytedocs_instance()
    if bytedocs is None:
        return HttpResponse(
            "ByteDocs not initialized. Please call setup_bytedocs() in your settings or urls.",
            status=500
        )

    handler = create_openapi_yaml_handler(bytedocs)
    return handler(request)


def ai_chat(request: HttpRequest) -> HttpResponse:
    """Handle AI chat requests"""
    bytedocs = get_bytedocs_instance()
    if bytedocs is None:
        return HttpResponse(
            "ByteDocs not initialized. Please call setup_bytedocs() in your settings or urls.",
            status=500
        )

    handler = create_chat_handler(bytedocs)
    return handler(request)



"""
ByteDocs Django - Alternative to Swagger/Scramble for Django
Beautiful API documentation with AI-powered features
"""

from .core.bytedocs import ByteDocs, setup_bytedocs, get_bytedocs_instance
from .core.types import (
    ByteDocsConfig,
    AuthConfig,
    UIConfig,
    AIConfig,
    AIFeatures,
    BaseURLOption,
    Schema,
    Parameter,
    ParameterLocation,
    RequestBody,
    ResponseDef,
    Endpoint,
    RouteInfo,
    Section,
)
from .core.config import load_config_from_env, validate_config, merge_configs

__version__ = "0.1.2"
__all__ = [
    "ByteDocs",
    "setup_bytedocs",
    "get_bytedocs_instance",
    "ByteDocsConfig",
    "AuthConfig",
    "UIConfig",
    "AIConfig",
    "AIFeatures",
    "BaseURLOption",
    "Schema",
    "Parameter",
    "ParameterLocation",
    "RequestBody",
    "ResponseDef",
    "Endpoint",
    "RouteInfo",
    "Section",
    "load_config_from_env",
    "validate_config",
    "merge_configs",
]



"""
ByteDocs Django - Main Documentation Engine
Based on ByteDocs Express implementation
"""

import json
from typing import Any, Dict, List, Optional
from dataclasses import asdict

from .types import (
    ByteDocsConfig,
    RouteInfo,
    Endpoint,
    Section,
    OpenAPISpec,
    OpenAPIServer,
    Parameter,
    ResponseDef,
    Schema,
)
from .config import load_config_from_env, validate_config, merge_configs
from ..parser.route_analyzer import RouteAnalyzer


class ByteDocs:
    """Main ByteDocs documentation class for Django"""

    def __init__(self, config: Optional[Dict[str, Any]] = None, urlconf=None):
        # Load from environment
        env_config = load_config_from_env()

        # Merge with user config
        if config:
            self.config = merge_configs(env_config, config)
        else:
            self.config = env_config

        # Validate config
        is_valid, errors = validate_config(self.config)
        if not is_valid:
            print(f"[ByteDocs] Configuration warnings: {errors}")

        self.urlconf = urlconf
        self.routes: List[RouteInfo] = []
        self.endpoints: List[Endpoint] = []
        self.sections: List[Section] = []
        self._detected = False

        # Add docs path to exclude paths
        if self.config.docs_path not in self.config.exclude_paths:
            self.config.exclude_paths.append(self.config.docs_path)

    def detect_routes(self, urlconf=None) -> None:
        """Detect routes from Django URLconf"""
        analyzer = RouteAnalyzer(urlconf or self.urlconf)
        all_routes = analyzer.get_all_routes()

        # Filter excluded paths and duplicates
        self.routes = []
        seen_routes = set()  # Track unique (method, path) combinations

        for route in all_routes:
            should_exclude = False

            # Skip excluded paths
            for exclude_path in self.config.exclude_paths:
                if route.path.startswith(exclude_path):
                    should_exclude = True
                    break

            # Skip DRF format suffix patterns (e.g., /api/items\.{format}/? or /api/{format})
            if '\\.{format}' in route.path or '.{format}' in route.path or route.path.endswith('/{format}'):
                should_exclude = True

            # Skip DRF API root views (e.g., /api/, /v1/, etc.) - they just list endpoints
            # Only exclude if it's exactly a root path (ends with / and has no resource name after)
            if route.path.count('/') == 2 and route.path.endswith('/'):
                # Paths like /api/ or /v1/ have exactly 2 slashes and end with /
                # But /api/users/ has 3 slashes, so won't be excluded
                if route.summary and 'root view' in route.summary.lower():
                    should_exclude = True

            # Skip regex patterns that leaked through
            if any(char in route.path for char in ['\\', '?', '*', '+', '[', ']']):
                should_exclude = True

            # Skip duplicate routes (same method + path)
            route_key = (route.method, route.path)
            if route_key in seen_routes:
                should_exclude = True

            if not should_exclude:
                self.routes.append(route)
                seen_routes.add(route_key)

        self._detected = True

        # Auto-generate documentation
        if self.config.auto_detect:
            self.generate()

    def add_route(self, route: RouteInfo) -> None:
        """Add route manually"""
        self.routes.append(route)
        self._detected = False

    def generate(self) -> None:
        """Generate documentation from routes"""
        self.endpoints = []
        self.sections = []

        # Convert routes to endpoints
        for route in self.routes:
            endpoint = self._route_to_endpoint(route)
            self.endpoints.append(endpoint)

        # Post-process: Copy request body schema from POST to PUT/PATCH
        self._copy_request_body_from_post()

        # Group endpoints into sections
        self.sections = self._group_endpoints_into_sections(self.endpoints)

    def _route_to_endpoint(self, route: RouteInfo) -> Endpoint:
        """Convert route to endpoint"""
        path = route.path
        method = route.method.upper()

        # Generate unique ID
        operation_id = f"{method.lower()}_{path.replace('/', '_').replace('{', '').replace('}', '')}"

        return Endpoint(
            path=path,
            method=method,
            summary=route.summary,
            description=route.description,
            tags=route.tags or self._infer_tags(path),
            parameters=route.parameters or [],
            request_body=route.request_body,
            responses=route.responses or self._get_default_responses(method),
            operation_id=operation_id,
        )

    def _infer_tags(self, path: str) -> List[str]:
        """Infer tags from path"""
        parts = [p for p in path.split("/") if p and not p.startswith("{")]
        if parts:
            tag = parts[0].replace("-", " ").replace("_", " ").title()
            return [tag]
        return ["Default"]

    def _get_default_responses(self, method: str) -> Dict[str, ResponseDef]:
        """Get default responses for method"""
        success_status = "201" if method.upper() == "POST" else "200"

        return {
            success_status: ResponseDef(
                description="Successful operation",
                content={
                    "application/json": {"schema": Schema(type="object")}
                },
            )
        }

    def _copy_request_body_from_post(self) -> None:
        """Copy request body schema from POST to PUT/PATCH endpoints"""
        # Create map of POST endpoints
        post_endpoints: Dict[str, Endpoint] = {}

        for endpoint in self.endpoints:
            if endpoint.method == "POST" and endpoint.request_body:
                post_endpoints[endpoint.path] = endpoint

        # Update PUT/PATCH endpoints
        for endpoint in self.endpoints:
            if endpoint.method in ["PUT", "PATCH"]:
                # Check if request body is empty
                has_empty_body = (
                    not endpoint.request_body
                    or not endpoint.request_body.content.get("application/json", {}).get(
                        "schema"
                    )
                )

                if has_empty_body:
                    # Find corresponding POST endpoint
                    base_path = endpoint.path
                    # Remove path parameters like /{id}
                    if "/" in base_path:
                        parts = base_path.split("/")
                        if parts[-1].startswith("{"):
                            base_path = "/".join(parts[:-1])

                    post_endpoint = post_endpoints.get(base_path)
                    if post_endpoint and post_endpoint.request_body:
                        print(
                            f"[ByteDocs] Copying request body from POST {base_path} to {endpoint.method} {endpoint.path}"
                        )
                        # Deep copy request body
                        endpoint.request_body = post_endpoint.request_body

    def _group_endpoints_into_sections(
        self, endpoints: List[Endpoint]
    ) -> List[Section]:
        """Group endpoints into sections by tags"""
        section_map: Dict[str, List[Endpoint]] = {}

        for endpoint in endpoints:
            section_name = endpoint.tags[0] if endpoint.tags else "Default"

            if section_name not in section_map:
                section_map[section_name] = []

            section_map[section_name].append(endpoint)

        sections = []
        for name, endpoints in section_map.items():
            sections.append(Section(name=name, endpoints=endpoints))

        return sections

    def get_documentation_data(self) -> Dict[str, Any]:
        """Get documentation data for UI"""
        # Transform sections to add IDs
        sections_with_ids = []
        for section in self.sections:
            endpoints_with_ids = []
            for endpoint in section.endpoints:
                endpoint_dict = self._to_dict(endpoint)
                endpoint_dict["id"] = endpoint.operation_id or f"{endpoint.method.lower()}_{endpoint.path}"

                # Convert snake_case to camelCase for UI compatibility
                if "request_body" in endpoint_dict:
                    endpoint_dict["requestBody"] = endpoint_dict.pop("request_body")
                if "operation_id" in endpoint_dict:
                    endpoint_dict["operationId"] = endpoint_dict.pop("operation_id")

                endpoints_with_ids.append(endpoint_dict)

            sections_with_ids.append(
                {"name": section.name, "endpoints": endpoints_with_ids}
            )

        # Convert base_urls to dict format (handle dataclass or dict)
        base_urls_list = []
        if self.config.base_urls:
            for url in self.config.base_urls:
                if hasattr(url, 'name'):  # dataclass
                    base_urls_list.append({"name": url.name, "url": url.url})
                else:  # dict
                    base_urls_list.append(url)
        elif self.config.base_url:
            base_urls_list = [{"name": "Default", "url": self.config.base_url}]

        return {
            "title": self.config.title,
            "version": self.config.version,
            "description": self.config.description,
            "baseURLs": base_urls_list,
            "endpoints": sections_with_ids,
        }

    def get_openapi_spec(self) -> Dict[str, Any]:
        """Get OpenAPI specification"""
        spec = {
            "openapi": "3.0.3",
            "info": {
                "title": self.config.title or "API Documentation",
                "version": self.config.version or "1.0.0",
                "description": self.config.description,
            },
            "paths": {},
        }

        # Add servers
        if self.config.base_urls:
            spec["servers"] = [
                {"url": base_url.url, "description": base_url.name}
                for base_url in self.config.base_urls
            ]
        elif self.config.base_url:
            spec["servers"] = [{"url": self.config.base_url}]

        # Add tags
        tags = set()
        for endpoint in self.endpoints:
            if endpoint.tags:
                for tag in endpoint.tags:
                    tags.add(tag)

        if tags:
            spec["tags"] = [{"name": tag} for tag in sorted(tags)]

        # Add paths
        for endpoint in self.endpoints:
            if endpoint.path not in spec["paths"]:
                spec["paths"][endpoint.path] = {}

            operation = {
                "summary": endpoint.summary,
                "description": endpoint.description,
                "tags": endpoint.tags,
                "parameters": [self._to_dict(p) for p in endpoint.parameters],
                "responses": {
                    status: self._to_dict(resp)
                    for status, resp in endpoint.responses.items()
                },
            }

            if endpoint.request_body:
                operation["requestBody"] = self._to_dict(endpoint.request_body)

            if endpoint.operation_id:
                operation["operationId"] = endpoint.operation_id

            if endpoint.deprecated:
                operation["deprecated"] = True

            spec["paths"][endpoint.path][endpoint.method.lower()] = operation

        return spec

    def handle_chat(self, message: str, endpoint_id: Optional[str] = None) -> Dict[str, Any]:
        """Handle AI chat request

        Args:
            message: User message
            endpoint_id: Optional endpoint ID for context

        Returns:
            AI response dict
        """
        if not self.config.ai_config or not self.config.ai_config.enabled:
            raise ValueError("AI features are not enabled")

        try:
            from ..ai import create_client, ChatRequest
            from ..ai.context_optimizer import get_optimizer

            # Create AI client
            ai_client = create_client(self.config.ai_config)

            # Get OpenAPI spec as context
            openapi_spec = self.get_openapi_spec()

            # Optimize context to reduce token usage (70-80% savings)
            optimizer = get_optimizer()

            # Get original size for comparison
            original_context = json.dumps(openapi_spec, indent=2)
            original_size = len(original_context)

            # Get optimized context
            optimized_context = optimizer.get_optimized_context(
                openapi_spec,
                endpoint_id=endpoint_id
            )
            optimized_size = len(optimized_context)

            # Log token savings
            savings = optimizer.estimate_token_savings(original_size, optimized_size)
            print(f"[ByteDocs] Context optimization - Tokens saved: {savings['tokens_saved']} ({savings['percentage_saved']})")

            # Find endpoint if endpoint_id provided
            endpoint_data = None
            if endpoint_id:
                for section in self.sections:
                    for endpoint in section.endpoints:
                        if endpoint.operation_id == endpoint_id:
                            endpoint_data = endpoint
                            break

            # Create chat request with optimized context
            ai_request = ChatRequest(
                message=message,
                context=optimized_context,
                endpoint=endpoint_data,
            )

            # Get response from AI (sync call for Django)
            import asyncio
            try:
                loop = asyncio.get_event_loop()
            except RuntimeError:
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)

            response = loop.run_until_complete(ai_client.chat(ai_request))

            return {
                "response": response.response,
                "provider": response.provider,
                "model": response.model,
                "tokensUsed": response.tokens_used,
                "tokensSaved": savings["tokens_saved"],
                "error": response.error,
            }

        except Exception as e:
            import traceback
            print(f"[ByteDocs] AI Chat error: {e}")
            traceback.print_exc()
            raise

    def get_routes(self) -> List[RouteInfo]:
        """Get all routes"""
        return self.routes.copy()

    def get_endpoints(self) -> List[Endpoint]:
        """Get all endpoints"""
        return self.endpoints.copy()

    def get_sections(self) -> List[Section]:
        """Get all sections"""
        return self.sections.copy()

    def get_config(self) -> ByteDocsConfig:
        """Get configuration"""
        return self.config

    def _to_dict(self, obj: Any) -> Any:
        """Convert dataclass/object to dict recursively"""
        if hasattr(obj, "__dataclass_fields__"):
            result = {}
            for field_name, field in obj.__dataclass_fields__.items():
                value = getattr(obj, field_name)
                # Convert field name from snake_case to camelCase for 'location' -> 'in'
                if field_name == "location" and hasattr(obj, "__class__") and obj.__class__.__name__ == "Parameter":
                    key = "in"
                    result[key] = value.value if hasattr(value, "value") else value
                # Convert snake_case to camelCase for OpenAPI schema properties
                elif field_name == "min_length":
                    key = "minLength"
                    if value is not None:
                        result[key] = self._to_dict(value)
                elif field_name == "max_length":
                    key = "maxLength"
                    if value is not None:
                        result[key] = self._to_dict(value)
                else:
                    key = field_name
                    if value is not None:
                        result[key] = self._to_dict(value)
            return result
        elif isinstance(obj, (list, tuple)):
            return [self._to_dict(item) for item in obj]
        elif isinstance(obj, dict):
            return {k: self._to_dict(v) for k, v in obj.items()}
        elif hasattr(obj, "value"):  # Enum
            return obj.value
        elif isinstance(obj, (str, int, float, bool, type(None))):
            # JSON-serializable primitives
            return obj
        else:
            # For non-serializable objects
            try:
                # Try json.dumps to see if it's serializable
                json.dumps(obj)
                return obj
            except (TypeError, ValueError):
                # Not serializable, return None
                return None


# Global ByteDocs instance
_bytedocs_instance: Optional[ByteDocs] = None


def setup_bytedocs(config: Optional[Dict[str, Any]] = None, urlconf=None) -> ByteDocs:
    """Setup ByteDocs for Django (convenience function)

    Args:
        config: Optional configuration dict
        urlconf: Optional URLconf module

    Returns:
        ByteDocs instance
    """
    global _bytedocs_instance
    _bytedocs_instance = ByteDocs(config, urlconf)
    return _bytedocs_instance


def get_bytedocs_instance() -> Optional[ByteDocs]:
    """Get global ByteDocs instance

    Returns:
        ByteDocs instance or None
    """
    return _bytedocs_instance



"""
AI Client Factory
Based on bytedocs-go factory pattern
"""

from typing import Dict, Callable
from .types import AIConfig, AIClient


# Registry of client factories
_client_factories: Dict[str, Callable[[AIConfig], AIClient]] = {}


def register_client_factory(provider: str, factory: Callable[[AIConfig], AIClient]) -> None:
    """Register a client factory for a provider"""
    _client_factories[provider] = factory


def create_client(config: AIConfig) -> AIClient:
    """Create AI client based on configuration"""
    if not config or not config.enabled:
        raise ValueError("AI configuration is not enabled")

    factory = _client_factories.get(config.provider)
    if not factory:
        raise ValueError(f"Unsupported AI provider: {config.provider}")

    return factory(config)


def get_available_providers() -> list:
    """Get list of available providers"""
    return list(_client_factories.keys())



"""
AI Provider Clients
Register all available providers
"""

from ..factory import register_client_factory
from .openai_client import OpenAIClient
from .gemini_client import GeminiClient
from .anthropic_client import AnthropicClient
from .openrouter_client import OpenRouterClient


# Register all providers
def _register_providers():
    """Register all available AI provider factories"""
    register_client_factory("openai", lambda config: OpenAIClient(config))
    register_client_factory("gemini", lambda config: GeminiClient(config))
    register_client_factory("anthropic", lambda config: AnthropicClient(config))
    register_client_factory("openrouter", lambda config: OpenRouterClient(config))


# Auto-register on import
_register_providers()


__all__ = [
    "OpenAIClient",
    "GeminiClient",
    "AnthropicClient",
    "OpenRouterClient",
]



"""
Context Optimizer for LLM Token Reduction
Implements JSON minification, context compression, and prompt optimization
"""

import json
from typing import Dict, Any, Optional


class ContextOptimizer:
    """Optimize context for LLM to reduce token usage"""

    def __init__(self):
        # Fields to keep in OpenAPI spec (remove unnecessary fields)
        self.essential_fields = {
            "paths": True,
            "info": ["title", "version"],
            "servers": True,
            "tags": True,
        }

    def minify_json(self, data: Dict[str, Any]) -> str:
        """Minify JSON by removing whitespace and formatting

        Token Savings: ~30-40%
        """
        return json.dumps(data, separators=(',', ':'), ensure_ascii=False)

    def compress_openapi_spec(self, spec: Dict[str, Any]) -> Dict[str, Any]:
        """Compress OpenAPI spec by removing non-essential fields

        Token Savings: ~40-60%
        """
        compressed = {}

        # Keep only essential info fields
        if "info" in spec:
            compressed["info"] = {
                "title": spec["info"].get("title", ""),
                "version": spec["info"].get("version", "")
            }

        # Keep servers (base URLs)
        if "servers" in spec:
            compressed["servers"] = spec["servers"]

        # Compress paths - this is the main content
        if "paths" in spec:
            compressed["paths"] = self._compress_paths(spec["paths"])

        # Keep tags for organization
        if "tags" in spec:
            compressed["tags"] = spec["tags"]

        return compressed

    def _compress_paths(self, paths: Dict[str, Any]) -> Dict[str, Any]:
        """Compress paths by removing verbose descriptions and examples

        Keep only:
        - Method
        - Summary (short description)
        - Parameters (name, in, required, type)
        - Request body schema (minimal)
        - Response status codes
        """
        compressed_paths = {}

        for path, methods in paths.items():
            compressed_paths[path] = {}

            for method, details in methods.items():
                if method.startswith("_"):  # Skip private fields
                    continue

                compressed_method = {}

                # Summary only (no long description)
                if "summary" in details:
                    compressed_method["s"] = details["summary"]  # Shortened key

                # Tags
                if "tags" in details:
                    compressed_method["t"] = details["tags"]

                # Parameters - minimal info
                if "parameters" in details and details["parameters"]:
                    compressed_method["p"] = [
                        {
                            "n": p.get("name"),  # name
                            "i": p.get("in"),     # in (location)
                            "r": p.get("required", False),  # required
                            "t": self._get_param_type(p)    # type
                        }
                        for p in details["parameters"]
                    ]

                # Request body - just schema type
                if "requestBody" in details:
                    req_body = details["requestBody"]
                    if "content" in req_body:
                        for content_type, content_data in req_body["content"].items():
                            if "schema" in content_data:
                                compressed_method["rb"] = self._compress_schema(
                                    content_data["schema"]
                                )
                                break  # Only take first content type

                # Responses - just status codes
                if "responses" in details:
                    compressed_method["rs"] = list(details["responses"].keys())

                compressed_paths[path][method] = compressed_method

        return compressed_paths

    def _get_param_type(self, param: Dict[str, Any]) -> str:
        """Extract parameter type from schema"""
        if "schema" in param:
            schema = param["schema"]
            if isinstance(schema, dict):
                return schema.get("type", "string")
        return "string"

    def _compress_schema(self, schema: Dict[str, Any]) -> Dict[str, Any]:
        """Compress schema to minimal representation"""
        if not isinstance(schema, dict):
            return {}

        compressed = {}

        # Type
        if "type" in schema:
            compressed["t"] = schema["type"]

        # Properties (for objects)
        if "properties" in schema:
            compressed["pr"] = {}
            for prop_name, prop_schema in schema["properties"].items():
                if isinstance(prop_schema, dict):
                    compressed["pr"][prop_name] = {
                        "t": prop_schema.get("type", "string")
                    }

        # Required fields
        if "required" in schema:
            compressed["r"] = schema["required"]

        return compressed

    def optimize_system_prompt(self, base_prompt: str) -> str:
        """Optimize system prompt to reduce tokens while maintaining effectiveness

        Token Savings: ~50-60%
        """
        # Shortened, more concise prompt
        optimized = """You're an API docs assistant. Use ONLY the provided OpenAPI spec.

Rules:
1. Only mention endpoints in the spec
2. Never invent endpoints/params
3. If endpoint doesn't exist, say so
4. Be concise
5. Match user's language

Format shortened:
- s: summary
- t: tags
- p: parameters (n:name, i:in, r:required, t:type)
- rb: request body
- rs: response statuses"""

        return optimized

    def get_optimized_context(
        self,
        openapi_spec: Dict[str, Any],
        endpoint_id: Optional[str] = None
    ) -> str:
        """Get fully optimized context for LLM

        Combines all optimization techniques:
        1. Compress spec (remove non-essential fields)
        2. Filter by endpoint if provided
        3. Minify JSON

        Total Token Savings: ~70-80%
        """
        # Step 1: Compress the spec
        compressed_spec = self.compress_openapi_spec(openapi_spec)

        # Step 2: If endpoint_id provided, filter to relevant paths only
        if endpoint_id and "paths" in compressed_spec:
            compressed_spec = self._filter_by_endpoint(compressed_spec, endpoint_id)

        # Step 3: Minify to remove whitespace
        minified = self.minify_json(compressed_spec)

        return minified

    def _filter_by_endpoint(
        self,
        spec: Dict[str, Any],
        endpoint_id: str
    ) -> Dict[str, Any]:
        """Filter spec to only include relevant endpoint and related ones

        For example, if asking about POST /users, include:
        - POST /users (exact match)
        - GET /users (same resource)
        - GET /users/{id} (related resource)
        """
        if "paths" not in spec:
            return spec

        # Extract path from endpoint_id (format: method_path_params)
        # e.g., "post_users" -> "/users"
        path_part = endpoint_id.lower().replace("_", "/")

        # Find matching and related paths
        filtered_paths = {}
        for path, methods in spec["paths"].items():
            path_normalized = path.lower().replace("{", "").replace("}", "")

            # Include if path matches or is related
            if path_part in path_normalized or path_normalized in path_part:
                filtered_paths[path] = methods

        # If we found related paths, use them; otherwise return all
        if filtered_paths:
            spec["paths"] = filtered_paths

        return spec

    def estimate_token_savings(
        self,
        original_size: int,
        optimized_size: int
    ) -> Dict[str, Any]:
        """Estimate token savings (rough approximation: 1 token ≈ 4 chars)"""
        original_tokens = original_size / 4
        optimized_tokens = optimized_size / 4
        saved_tokens = original_tokens - optimized_tokens
        percentage = (saved_tokens / original_tokens * 100) if original_tokens > 0 else 0

        return {
            "original_chars": original_size,
            "optimized_chars": optimized_size,
            "original_tokens_est": int(original_tokens),
            "optimized_tokens_est": int(optimized_tokens),
            "tokens_saved": int(saved_tokens),
            "percentage_saved": f"{percentage:.1f}%"
        }


# Global optimizer instance
_optimizer = ContextOptimizer()


def get_optimizer() -> ContextOptimizer:
    """Get global context optimizer instance"""
    return _optimizer



"""
Google Gemini Provider Client
Based on bytedocs-go Gemini implementation
"""

import os
from typing import Optional
import google.generativeai as genai

from ..types import AIConfig, AIClient, ChatRequest, ChatResponse
from ..context_optimizer import get_optimizer


class GeminiClient(AIClient):
    """Google Gemini client implementation"""

    def __init__(self, config: AIConfig):
        self.config = config

        # Get API key from config or environment
        api_key = config.api_key or os.getenv("GEMINI_API_KEY")
        if not api_key:
            raise ValueError(
                "Gemini API key is required (set api_key in config or GEMINI_API_KEY environment variable)"
            )

        # Configure Gemini
        genai.configure(api_key=api_key)

        # Default model
        self.model_name = config.features.model or "gemini-2.0-flash-exp"

        # Create model
        generation_config = {
            "temperature": config.features.temperature,
        }
        if config.features.max_tokens > 0:
            generation_config["max_output_tokens"] = config.features.max_tokens

        self.model = genai.GenerativeModel(
            model_name=self.model_name,
            generation_config=generation_config,
        )

    async def chat(self, request: ChatRequest) -> ChatResponse:
        """Send chat request to Gemini"""
        try:
            # Build full prompt (Gemini doesn't have separate system/user messages in the same way)
            full_prompt = self._build_system_prompt(request) + f"\n\nUser: {request.message}"

            # Make API call
            response = await self.model.generate_content_async(full_prompt)

            # Extract response
            if not response.text:
                raise ValueError("No response content returned")

            # Get tokens used (if available)
            tokens_used = 0
            if hasattr(response, 'usage_metadata') and response.usage_metadata:
                tokens_used = response.usage_metadata.total_token_count

            return ChatResponse(
                response=response.text,
                provider=self.get_provider(),
                model=self.model_name,
                tokens_used=tokens_used,
            )

        except Exception as e:
            return ChatResponse(
                response="",
                provider=self.get_provider(),
                model=self.model_name,
                error=str(e),
            )

    def get_provider(self) -> str:
        """Get provider name"""
        return "gemini"

    def get_model(self) -> str:
        """Get model name"""
        return self.model_name

    def _build_system_prompt(self, request: ChatRequest) -> str:
        """Build optimized system prompt with context"""
        optimizer = get_optimizer()

        # Use optimized prompt
        base_prompt = optimizer.optimize_system_prompt("")

        # Add API context (OpenAPI JSON) - already optimized
        if request.context:
            base_prompt += f"\n\nAPI Spec:\n{request.context}"

        # Add endpoint context if provided
        if request.endpoint:
            base_prompt += "\n\nFocused endpoint provided. Prioritize info about this endpoint."

        return base_prompt



"""
ByteDocs Django - Template Loader
Load and render HTML templates
"""

import os
from typing import Dict, Any, Optional
from pathlib import Path


class TemplateLoader:
    """Simple template loader with caching"""

    def __init__(self):
        self.cache: Dict[str, str] = {}
        self.template_dir = Path(__file__).parent / "templates"

    def load(self, template_name: str) -> str:
        """Load template file with caching"""
        if template_name in self.cache:
            return self.cache[template_name]

        template_path = self.template_dir / template_name

        if not template_path.exists():
            raise FileNotFoundError(f"Template not found: {template_name}")

        with open(template_path, "r", encoding="utf-8") as f:
            content = f.read()

        self.cache[template_name] = content
        return content

    def render(self, template: str, data: Dict[str, Any]) -> str:
        """Simple template rendering - replace placeholders"""
        rendered = template

        # Replace __PLACEHOLDER__ style placeholders
        for key, value in data.items():
            placeholder = f"__BYTEDOCS_{key.upper()}__"
            rendered = rendered.replace(placeholder, str(value) if value is not None else "")

        return rendered

    def clear_cache(self):
        """Clear template cache"""
        self.cache.clear()


# Global template loader instance
_loader = TemplateLoader()


def load_template(template_name: str) -> str:
    """Load template file"""
    return _loader.load(template_name)


def render_template(template: str, data: Dict[str, Any]) -> str:
    """Render template with data"""
    return _loader.render(template, data)


def clear_template_cache():
    """Clear template cache"""
    _loader.clear_cache()



"""
AI Module - Multi-provider AI support for ByteDocs
Based on bytedocs-go AI implementation
"""

from .types import (
    AIConfig,
    AIFeatures,
    AIClient,
    ChatRequest,
    ChatResponse,
    ClientFactory,
)
from .factory import (
    register_client_factory,
    create_client,
    get_available_providers,
)

# Import providers to trigger registration
from . import providers  # noqa: F401


__all__ = [
    # Types
    "AIConfig",
    "AIFeatures",
    "AIClient",
    "ChatRequest",
    "ChatResponse",
    "ClientFactory",
    # Factory functions
    "register_client_factory",
    "create_client",
    "get_available_providers",
]



"""
OpenRouter Provider Client
Based on bytedocs-go OpenRouter implementation
"""

import os
from typing import Optional
from openai import AsyncOpenAI

from ..types import AIConfig, AIClient, ChatRequest, ChatResponse
from ..context_optimizer import get_optimizer


class OpenRouterClient(AIClient):
    """OpenRouter client implementation (OpenAI-compatible)"""

    def __init__(self, config: AIConfig):
        self.config = config

        # Get API key from config or environment
        api_key = config.api_key or os.getenv("OPENROUTER_API_KEY")
        if not api_key:
            raise ValueError(
                "OpenRouter API key is required (set api_key in config or OPENROUTER_API_KEY environment variable)"
            )

        # Create OpenAI-compatible client with OpenRouter base URL
        self.client = AsyncOpenAI(
            api_key=api_key,
            base_url="https://openrouter.ai/api/v1",
        )

        # Default model
        self.model = config.features.model or "openai/gpt-3.5-turbo"

    async def chat(self, request: ChatRequest) -> ChatResponse:
        """Send chat request to OpenRouter"""
        try:
            # Build messages
            messages = [
                {"role": "system", "content": self._build_system_prompt(request)},
                {"role": "user", "content": request.message},
            ]

            # Prepare kwargs
            kwargs = {
                "model": self.model,
                "messages": messages,
            }

            # Add optional parameters
            if self.config.features.max_tokens > 0:
                kwargs["max_tokens"] = self.config.features.max_tokens
            if self.config.features.max_completion_tokens > 0:
                kwargs["max_completion_tokens"] = self.config.features.max_completion_tokens
            if self.config.features.temperature > 0:
                kwargs["temperature"] = self.config.features.temperature

            # Make API call
            response = await self.client.chat.completions.create(**kwargs)

            # Extract response
            if not response.choices:
                raise ValueError("No response choices returned")

            content = response.choices[0].message.content or ""
            tokens_used = response.usage.total_tokens if response.usage else 0

            return ChatResponse(
                response=content,
                provider=self.get_provider(),
                model=response.model,
                tokens_used=tokens_used,
            )

        except Exception as e:
            return ChatResponse(
                response="",
                provider=self.get_provider(),
                model=self.model,
                error=str(e),
            )

    def get_provider(self) -> str:
        """Get provider name"""
        return "openrouter"

    def get_model(self) -> str:
        """Get model name"""
        return self.model

    def _build_system_prompt(self, request: ChatRequest) -> str:
        """Build optimized system prompt with context"""
        optimizer = get_optimizer()

        # Use optimized prompt
        base_prompt = optimizer.optimize_system_prompt("")

        # Add API context (OpenAPI JSON) - already optimized
        if request.context:
            base_prompt += f"\n\nAPI Spec:\n{request.context}"

        # Add endpoint context if provided
        if request.endpoint:
            base_prompt += "\n\nFocused endpoint provided. Prioritize info about this endpoint."

        return base_prompt



"""
ByteDocs Django - Route Analyzer
Analyze Django URLconf and extract route information
"""

import inspect
import re
from typing import List, Dict, Any, Optional, Callable, get_type_hints, Union
from dataclasses import asdict
from django.urls import URLPattern, URLResolver, get_resolver
from django.urls.resolvers import RoutePattern, RegexPattern
from django.views import View

from ..core.types import RouteInfo, Parameter, ParameterLocation, Schema, RequestBody, ResponseDef
from .schema_extractor import SchemaExtractor
from .ast_analyzer import enhance_schema_with_ast, get_handler_source_code


class RouteAnalyzer:
    """Analyze Django routes and extract documentation"""

    def __init__(self, urlconf=None):
        """Initialize route analyzer

        Args:
            urlconf: Django URLconf module (default: ROOT_URLCONF from settings)
        """
        self.urlconf = urlconf
        self.resolver = get_resolver(urlconf)

    def get_all_routes(self) -> List[RouteInfo]:
        """Get all routes from Django URLconf

        Returns:
            List of RouteInfo objects
        """
        routes = []
        self._extract_routes(self.resolver.url_patterns, routes, prefix="")
        return routes

    def _extract_routes(
        self,
        url_patterns: List[Union[URLPattern, URLResolver]],
        routes: List[RouteInfo],
        prefix: str = "",
    ) -> None:
        """Recursively extract routes from URL patterns

        Args:
            url_patterns: List of URL patterns or resolvers
            routes: List to append RouteInfo objects to
            prefix: URL prefix for nested patterns
        """
        for pattern in url_patterns:
            if isinstance(pattern, URLResolver):
                # Nested URL patterns (include())
                new_prefix = prefix + str(pattern.pattern)
                self._extract_routes(pattern.url_patterns, routes, prefix=new_prefix)
            elif isinstance(pattern, URLPattern):
                # Single URL pattern
                path = prefix + str(pattern.pattern)
                callback = pattern.callback

                # Skip if no callback (shouldn't happen)
                if callback is None:
                    continue

                # Get view function
                view_func = self._get_view_function(callback)

                # Extract route info (returns list of routes for different HTTP methods)
                route_infos = self._analyze_view(view_func, path, pattern)
                if route_infos:
                    routes.extend(route_infos)

    def _get_view_function(self, callback: Callable) -> Callable:
        """Get the actual view function from callback

        Args:
            callback: View callback (function or class-based view)

        Returns:
            View function
        """
        # Handle class-based views
        if inspect.isclass(callback) and issubclass(callback, View):
            # For CBV, we'll use the dispatch method
            return callback.as_view()

        # Handle function-based views
        return callback

    def _analyze_view(
        self,
        view_func: Callable,
        path: str,
        pattern: URLPattern,
    ) -> List[RouteInfo]:
        """Analyze a view function and extract route information

        Args:
            view_func: View function to analyze
            path: URL path
            pattern: URL pattern object

        Returns:
            List of RouteInfo objects (one per HTTP method)
        """
        # Determine HTTP methods
        methods = self._get_http_methods(view_func)

        # Extract parameters from URL pattern
        parameters = self._extract_path_parameters(pattern)

        # Get docstring
        class_docstring = inspect.getdoc(view_func) or ""

        # Get tags from view function
        tags = self._get_tags(view_func, path)

        # Extract request body and responses from DRF ViewSet
        request_body = self._extract_request_body(view_func, path)
        responses = self._extract_responses(view_func, path)

        # Enhance responses with AST analysis
        source_code = get_handler_source_code(view_func)
        if source_code:
            responses = enhance_schema_with_ast(responses, view_func, source_code)

        # Parse DRF ViewSet docstring format (action-based)
        action_docs = self._parse_drf_docstring(class_docstring)

        # Create RouteInfo for each HTTP method
        routes = []
        for method in methods:
            # Get action name from view_func.actions if available
            action_name = None
            if hasattr(view_func, 'actions') and method.lower() in view_func.actions:
                action_name = view_func.actions[method.lower()]

            # Get summary and description for this specific action
            summary = None
            description = None

            # For custom actions (not standard CRUD), get docstring from the action method directly
            if action_name and action_name not in ['list', 'create', 'retrieve', 'update', 'partial_update', 'destroy']:
                # This is a custom action, get docstring from the action method
                if hasattr(view_func, 'cls'):
                    view_class = view_func.cls
                    if hasattr(view_class, action_name):
                        action_method = getattr(view_class, action_name)
                        action_docstring = inspect.getdoc(action_method) or ""
                        if action_docstring:
                            summary, description = self._parse_docstring(action_docstring)

            # If not found from custom action, try standard action docs
            if not summary and action_name and action_name in action_docs:
                summary = action_docs[action_name].get('summary')
                description = action_docs[action_name].get('description')

            # Final fallback to parsing class docstring
            if not summary:
                summary, description = self._parse_docstring(class_docstring)

            route_info = RouteInfo(
                method=method,
                path=self._convert_path_pattern(path),
                handler=view_func,
                summary=summary or self._generate_summary(method, path),
                description=description or "",  # OpenAPI requires string, not None
                parameters=parameters[:],  # Copy parameters
                request_body=request_body if method in ['POST', 'PUT', 'PATCH'] else None,
                responses=responses,
                tags=tags,
            )
            routes.append(route_info)

        return routes

    def _get_http_methods(self, view_func: Callable) -> List[str]:
        """Get HTTP methods supported by view

        Args:
            view_func: View function

        Returns:
            List of HTTP methods (uppercase)
        """
        # Check if it's a DRF view with actions attribute (from router)
        if hasattr(view_func, 'actions'):
            # DRF router sets actions like {'get': 'list', 'post': 'create'}
            return [method.upper() for method in view_func.actions.keys()]

        # Check if view has explicit allowed methods
        if hasattr(view_func, 'allowed_methods'):
            return [m.upper() for m in view_func.allowed_methods]

        # Check if it's a class-based view
        if hasattr(view_func, 'view_class'):
            view_class = view_func.view_class
            if hasattr(view_class, 'http_method_names'):
                # Get allowed methods from class
                allowed_methods = []
                for method in view_class.http_method_names:
                    if hasattr(view_class, method):
                        allowed_methods.append(method.upper())
                return allowed_methods

        # Default to GET only for safety
        return ['GET']

    def _extract_path_parameters(self, pattern: URLPattern) -> List[Parameter]:
        """Extract path parameters from URL pattern

        Args:
            pattern: URL pattern object

        Returns:
            List of Parameter objects
        """
        parameters = []

        # Get pattern regex
        if isinstance(pattern.pattern, RoutePattern):
            # Django 2.0+ path() syntax
            # Extract parameters from <type:name> syntax
            regex = pattern.pattern._route
            matches = re.findall(r'<(\w+:)?(\w+)>', regex)
            for match in matches:
                param_type, param_name = match
                param_type = param_type.rstrip(':') if param_type else 'str'

                # Map Django parameter types to OpenAPI types
                schema = self._get_schema_for_param_type(param_type)

                parameters.append(Parameter(
                    name=param_name,
                    location=ParameterLocation.PATH,
                    schema=schema,
                    required=True,
                ))
        elif isinstance(pattern.pattern, RegexPattern):
            # Old re_path() syntax
            # Extract named groups from regex
            regex = pattern.pattern.regex.pattern
            matches = re.findall(r'\?P<(\w+)>', regex)
            for param_name in matches:
                parameters.append(Parameter(
                    name=param_name,
                    location=ParameterLocation.PATH,
                    schema=Schema(type="string"),
                    required=True,
                ))

        return parameters

    def _get_schema_for_param_type(self, param_type: str) -> Schema:
        """Get OpenAPI schema for Django path parameter type

        Args:
            param_type: Django parameter type (int, str, slug, uuid, path)

        Returns:
            Schema object
        """
        type_mapping = {
            'int': Schema(type='integer', format='int32'),
            'str': Schema(type='string'),
            'slug': Schema(type='string', pattern=r'^[-a-zA-Z0-9_]+$'),
            'uuid': Schema(type='string', format='uuid'),
            'path': Schema(type='string'),
        }
        return type_mapping.get(param_type, Schema(type='string'))

    def _convert_path_pattern(self, path: str) -> str:
        """Convert Django path pattern to OpenAPI path format

        Args:
            path: Django path pattern

        Returns:
            OpenAPI path format (e.g., /users/{id})
        """
        converted = path

        # Convert regex named groups (?P<name>pattern) to {name}
        # This handles DRF router patterns like (?P<pk>[^/.]+)
        converted = re.sub(r'\(\?P<(\w+)>[^\)]+\)', r'{\1}', converted)

        # Convert Django <type:name> to OpenAPI {name}
        converted = re.sub(r'<(\w+):(\w+)>', r'{\2}', converted)
        # Convert Django <name> to OpenAPI {name}
        converted = re.sub(r'<(\w+)>', r'{\1}', converted)

        # Remove all regex anchors and special chars
        converted = converted.replace('^', '')
        converted = converted.replace('$', '')

        # Ensure path starts with /
        if not converted.startswith('/'):
            converted = '/' + converted
        # Keep trailing slashes for Django compatibility (DRF requires them by default)

        return converted

    def _parse_drf_docstring(self, docstring: str) -> Dict[str, Dict[str, str]]:
        """Parse DRF ViewSet docstring with action-specific docs

        Args:
            docstring: ViewSet docstring

        Returns:
            Dict mapping action names to their documentation
            Example: {'list': {'summary': 'Get all items', 'description': '...'}}
        """
        if not docstring:
            return {}

        action_docs = {}
        lines = docstring.strip().split('\n')

        current_action = None
        current_lines = []

        for line in lines:
            # Check if line is an action declaration (ends with :)
            stripped = line.strip()
            if stripped and stripped.endswith(':') and not stripped.startswith('-'):
                # Save previous action if exists
                if current_action and current_lines:
                    text = '\n'.join(current_lines).strip()
                    # First line is summary, rest is description
                    parts = text.split('\n', 1)
                    action_docs[current_action] = {
                        'summary': parts[0].strip() if parts else '',
                        'description': parts[1].strip() if len(parts) > 1 else None
                    }

                # Start new action
                current_action = stripped.rstrip(':').strip()
                current_lines = []
            elif current_action:
                # Add line to current action
                if stripped:
                    current_lines.append(stripped)

        # Save last action
        if current_action and current_lines:
            text = '\n'.join(current_lines).strip()
            parts = text.split('\n', 1)
            action_docs[current_action] = {
                'summary': parts[0].strip() if parts else '',
                'description': parts[1].strip() if len(parts) > 1 else None
            }

        return action_docs

    def _parse_docstring(self, docstring: str) -> tuple[Optional[str], Optional[str]]:
        """Parse docstring into summary and description

        Args:
            docstring: Function docstring

        Returns:
            Tuple of (summary, description)
        """
        if not docstring:
            return None, None

        lines = docstring.strip().split('\n')
        if not lines:
            return None, None

        # First line is summary
        summary = lines[0].strip()

        # Rest is description
        description = None
        if len(lines) > 1:
            desc_lines = [line.strip() for line in lines[1:] if line.strip()]
            if desc_lines:
                description = '\n'.join(desc_lines)

        return summary, description

    def _generate_summary(self, method: str, path: str) -> str:
        """Generate summary from method and path

        Args:
            method: HTTP method
            path: URL path

        Returns:
            Generated summary
        """
        # Remove parameters and convert to words
        clean_path = re.sub(r'\{[^}]+\}', '', path)
        parts = [p for p in clean_path.split('/') if p]

        if not parts:
            resource = "root"
        else:
            resource = parts[-1].replace('_', ' ').replace('-', ' ')

        method_action = {
            'GET': 'Get',
            'POST': 'Create',
            'PUT': 'Update',
            'PATCH': 'Partially update',
            'DELETE': 'Delete',
        }.get(method, method.title())

        return f"{method_action} {resource}"

    def _get_tags(self, view_func: Callable, path: str) -> List[str]:
        """Get tags for endpoint

        Args:
            view_func: View function
            path: URL path

        Returns:
            List of tags
        """
        # Check if view has explicit tags
        if hasattr(view_func, 'bytedocs_tags'):
            return view_func.bytedocs_tags

        # Check if view class has tags
        if hasattr(view_func, 'view_class') and hasattr(view_func.view_class, 'bytedocs_tags'):
            return view_func.view_class.bytedocs_tags

        # Generate tag from path
        parts = [p for p in path.split('/') if p and not p.startswith('{')]
        if parts:
            tag = parts[0].replace('_', ' ').replace('-', ' ').title()
            return [tag]

        return ['Default']

    def _extract_request_body(self, view_func: Callable, path: str) -> Optional[RequestBody]:
        """Extract request body from DRF ViewSet

        Args:
            view_func: View function
            path: URL path

        Returns:
            RequestBody object or None
        """
        # Check if view has serializer_class (DRF ViewSet)
        serializer_class = None

        if hasattr(view_func, 'cls'):
            # DRF ViewSet
            view_class = view_func.cls
            if hasattr(view_class, 'serializer_class'):
                serializer_class = view_class.serializer_class
            elif hasattr(view_class, 'get_serializer_class'):
                try:
                    # Try to get serializer class
                    serializer_class = view_class().get_serializer_class()
                except Exception:
                    pass
        elif hasattr(view_func, 'view_class'):
            view_class = view_func.view_class
            if hasattr(view_class, 'serializer_class'):
                serializer_class = view_class.serializer_class

        if serializer_class:
            schema = SchemaExtractor.extract_from_serializer(serializer_class)
            return RequestBody(
                description="Request body",
                required=True,
                content={
                    "application/json": {
                        "schema": schema,
                    }
                },
            )

        return None

    def _extract_responses(self, view_func: Callable, path: str) -> Dict[str, ResponseDef]:
        """Extract responses from DRF ViewSet

        Args:
            view_func: View function
            path: URL path

        Returns:
            Dict of status code to ResponseDef
        """
        responses = {}

        # Check if view has serializer_class (DRF ViewSet)
        serializer_class = None

        if hasattr(view_func, 'cls'):
            # DRF ViewSet
            view_class = view_func.cls
            if hasattr(view_class, 'serializer_class'):
                serializer_class = view_class.serializer_class
            elif hasattr(view_class, 'get_serializer_class'):
                try:
                    serializer_class = view_class().get_serializer_class()
                except Exception:
                    pass
        elif hasattr(view_func, 'view_class'):
            view_class = view_func.view_class
            if hasattr(view_class, 'serializer_class'):
                serializer_class = view_class.serializer_class

        if serializer_class:
            schema = SchemaExtractor.extract_from_serializer(serializer_class)
            responses['200'] = ResponseDef(
                description="Successful operation",
                content={
                    "application/json": {
                        "schema": schema,
                    }
                },
            )
        else:
            # Default response
            responses['200'] = ResponseDef(
                description="Successful operation",
                content={
                    "application/json": {
                        "schema": Schema(type="object"),
                    }
                },
            )

        return responses


def get_routes(urlconf=None) -> List[RouteInfo]:
    """Get all routes from Django URLconf

    Args:
        urlconf: Django URLconf module (default: ROOT_URLCONF from settings)

    Returns:
        List of RouteInfo objects
    """
    analyzer = RouteAnalyzer(urlconf)
    return analyzer.get_all_routes()



"""
ByteDocs Django - Setup Configuration
"""

from setuptools import setup, find_packages
import os

# Read README
def read_file(filename):
    filepath = os.path.join(os.path.dirname(__file__), filename)
    if os.path.exists(filepath):
        with open(filepath, 'r', encoding='utf-8') as f:
            return f.read()
    return ''

setup(
    name='bytedocs-django',
    version='0.1.2',
    author='ByteDocs Team',
    author_email='bytedocs.dev@gmail.com',
    description='Alternative to Swagger with better design, auto-detection, and AI integration',
    long_description=read_file('README.md'),
    long_description_content_type='text/markdown',
    url='https://github.com/bytedocs/bytedocs-django',
    packages=find_packages(exclude=['examples', 'tests', 'venv']),
    include_package_data=True,
    package_data={
        'bytedocs_django': [
            'ui/templates/*.html',
            'ui/templates/auth/*.html',
        ],
    },
    install_requires=[
        'Django>=3.2',
        'pyyaml>=6.0',
        'python-dotenv>=1.0.0',
    ],
    extras_require={
        'ai': [
            'openai>=2.0.0',
            'google-generativeai>=0.8.0',
            'anthropic>=0.70.0',
        ],
        'drf': [
            'djangorestframework>=3.14.0',
        ],
        'dev': [
            'pytest>=7.0.0',
            'pytest-django>=4.5.0',
            'black>=23.0.0',
            'flake8>=6.0.0',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
        'Topic :: Software Development :: Documentation',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires='>=3.8',
    keywords='django api documentation swagger openapi rest-framework bytedocs',
    project_urls={
        'Bug Reports': 'https://github.com/bytedocs/bytedocs-django/issues',
        'Source': 'https://github.com/bytedocs/bytedocs-django',
        'Documentation': 'https://docs.bytedocs.com',
    },
)



"""
ByteDocs Django - Schema Extractor
Extract schema information from DRF serializers, Django models, and Pydantic models
"""

import inspect
from typing import Any, Dict, List, Optional, get_type_hints, Union
from dataclasses import asdict, is_dataclass

from ..core.types import Schema, Parameter, ParameterLocation, RequestBody, ResponseDef


class SchemaExtractor:
    """Extract schemas from various Django model types"""

    @staticmethod
    def extract_from_serializer(serializer_class) -> Schema:
        """Extract schema from DRF serializer

        Args:
            serializer_class: DRF Serializer class

        Returns:
            Schema object
        """
        try:
            from rest_framework import serializers
        except ImportError:
            return Schema(type="object")

        if not inspect.isclass(serializer_class) or not issubclass(serializer_class, serializers.Serializer):
            return Schema(type="object")

        # Create serializer instance
        try:
            serializer = serializer_class()
        except Exception:
            # If instantiation fails, return basic schema
            return Schema(type="object")

        properties = {}
        required = []

        # Extract fields from serializer
        for field_name, field in serializer.fields.items():
            # Get field schema
            field_schema = SchemaExtractor._extract_field_schema(field)
            properties[field_name] = field_schema

            # Check if required
            if field.required:
                required.append(field_name)

        return Schema(
            type="object",
            properties=properties,
            required=required if required else None,
        )

    @staticmethod
    def _extract_field_schema(field) -> Schema:
        """Extract schema from DRF field

        Args:
            field: DRF field instance

        Returns:
            Schema object
        """
        try:
            from rest_framework import serializers
        except ImportError:
            return Schema(type="string")

        # Map field types to OpenAPI types
        type_mapping = {
            serializers.IntegerField: ("integer", "int32"),
            serializers.FloatField: ("number", "float"),
            serializers.DecimalField: ("number", "double"),
            serializers.BooleanField: ("boolean", None),
            serializers.CharField: ("string", None),
            serializers.EmailField: ("string", "email"),
            serializers.URLField: ("string", "uri"),
            serializers.UUIDField: ("string", "uuid"),
            serializers.DateField: ("string", "date"),
            serializers.DateTimeField: ("string", "date-time"),
            serializers.TimeField: ("string", "time"),
            serializers.JSONField: ("object", None),
        }

        # Get base type
        field_type = type(field)
        openapi_type, openapi_format = type_mapping.get(field_type, ("string", None))

        schema = Schema(
            type=openapi_type,
            format=openapi_format,
            description=field.help_text if hasattr(field, 'help_text') else None,
            default=field.default if hasattr(field, 'default') and field.default is not serializers.empty else None,
        )

        # Handle list fields
        if isinstance(field, serializers.ListField):
            if hasattr(field, 'child'):
                schema = Schema(
                    type="array",
                    items=SchemaExtractor._extract_field_schema(field.child),
                )

        # Handle nested serializers
        elif isinstance(field, serializers.Serializer):
            schema = SchemaExtractor.extract_from_serializer(field.__class__)

        # Handle choice fields
        elif isinstance(field, serializers.ChoiceField):
            choices = list(field.choices.keys()) if hasattr(field, 'choices') else None
            schema.enum = choices

        # Handle min/max validators
        if hasattr(field, 'min_value'):
            schema.minimum = field.min_value
        if hasattr(field, 'max_value'):
            schema.maximum = field.max_value

        # Handle min/max length validators
        if hasattr(field, 'min_length'):
            schema.min_length = field.min_length
        if hasattr(field, 'max_length'):
            schema.max_length = field.max_length

        return schema

    @staticmethod
    def extract_from_model(model_class) -> Schema:
        """Extract schema from Django model

        Args:
            model_class: Django model class

        Returns:
            Schema object
        """
        try:
            from django.db import models
        except ImportError:
            return Schema(type="object")

        if not inspect.isclass(model_class) or not issubclass(model_class, models.Model):
            return Schema(type="object")

        properties = {}
        required = []

        # Extract fields from model
        for field in model_class._meta.get_fields():
            if hasattr(field, 'name'):
                field_name = field.name
                field_schema = SchemaExtractor._extract_model_field_schema(field)
                properties[field_name] = field_schema

                # Check if required
                if hasattr(field, 'null') and not field.null and hasattr(field, 'blank') and not field.blank:
                    required.append(field_name)

        return Schema(
            type="object",
            properties=properties,
            required=required if required else None,
        )

    @staticmethod
    def _extract_model_field_schema(field) -> Schema:
        """Extract schema from Django model field

        Args:
            field: Django model field instance

        Returns:
            Schema object
        """
        try:
            from django.db import models
        except ImportError:
            return Schema(type="string")

        # Map field types to OpenAPI types
        type_mapping = {
            models.IntegerField: ("integer", "int32"),
            models.BigIntegerField: ("integer", "int64"),
            models.SmallIntegerField: ("integer", "int32"),
            models.FloatField: ("number", "float"),
            models.DecimalField: ("number", "double"),
            models.BooleanField: ("boolean", None),
            models.CharField: ("string", None),
            models.TextField: ("string", None),
            models.EmailField: ("string", "email"),
            models.URLField: ("string", "uri"),
            models.UUIDField: ("string", "uuid"),
            models.DateField: ("string", "date"),
            models.DateTimeField: ("string", "date-time"),
            models.TimeField: ("string", "time"),
            models.JSONField: ("object", None),
        }

        # Get base type
        field_type = type(field)
        openapi_type, openapi_format = type_mapping.get(field_type, ("string", None))

        schema = Schema(
            type=openapi_type,
            format=openapi_format,
            description=field.help_text if hasattr(field, 'help_text') else None,
            default=field.default if hasattr(field, 'default') and field.default is not models.NOT_PROVIDED else None,
        )

        # Handle max length
        if hasattr(field, 'max_length') and field.max_length:
            schema.max_length = field.max_length

        return schema

    @staticmethod
    def extract_from_pydantic(model_class) -> Schema:
        """Extract schema from Pydantic model (for django-ninja)

        Args:
            model_class: Pydantic model class

        Returns:
            Schema object
        """
        try:
            from pydantic import BaseModel
        except ImportError:
            return Schema(type="object")

        if not inspect.isclass(model_class) or not issubclass(model_class, BaseModel):
            return Schema(type="object")

        # Get model schema from Pydantic
        try:
            pydantic_schema = model_class.model_json_schema()
            return SchemaExtractor._convert_pydantic_schema(pydantic_schema)
        except Exception:
            return Schema(type="object")

    @staticmethod
    def _convert_pydantic_schema(pydantic_schema: Dict[str, Any]) -> Schema:
        """Convert Pydantic JSON schema to our Schema format

        Args:
            pydantic_schema: Pydantic JSON schema dict

        Returns:
            Schema object
        """
        schema_type = pydantic_schema.get("type", "object")

        # Handle object type
        if schema_type == "object":
            properties = {}
            if "properties" in pydantic_schema:
                for prop_name, prop_schema in pydantic_schema["properties"].items():
                    properties[prop_name] = SchemaExtractor._convert_pydantic_schema(prop_schema)

            return Schema(
                type="object",
                properties=properties if properties else None,
                required=pydantic_schema.get("required"),
                description=pydantic_schema.get("description"),
            )

        # Handle array type
        elif schema_type == "array":
            items = None
            if "items" in pydantic_schema:
                items = SchemaExtractor._convert_pydantic_schema(pydantic_schema["items"])

            return Schema(
                type="array",
                items=items,
                description=pydantic_schema.get("description"),
            )

        # Handle primitive types
        else:
            return Schema(
                type=schema_type,
                format=pydantic_schema.get("format"),
                description=pydantic_schema.get("description"),
                enum=pydantic_schema.get("enum"),
                default=pydantic_schema.get("default"),
                minimum=pydantic_schema.get("minimum"),
                maximum=pydantic_schema.get("maximum"),
                min_length=pydantic_schema.get("minLength"),
                max_length=pydantic_schema.get("maxLength"),
                pattern=pydantic_schema.get("pattern"),
            )

    @staticmethod
    def extract_from_type(type_hint: Any) -> Schema:
        """Extract schema from Python type hint

        Args:
            type_hint: Python type hint

        Returns:
            Schema object
        """
        # Handle None
        if type_hint is None or type_hint is type(None):
            return Schema(type="null")

        # Handle basic types
        if type_hint == int:
            return Schema(type="integer", format="int32")
        elif type_hint == float:
            return Schema(type="number", format="float")
        elif type_hint == bool:
            return Schema(type="boolean")
        elif type_hint == str:
            return Schema(type="string")
        elif type_hint == dict:
            return Schema(type="object")
        elif type_hint == list:
            return Schema(type="array", items=Schema(type="string"))

        # Handle Optional, Union, List, Dict
        origin = getattr(type_hint, '__origin__', None)
        if origin is Union:
            # Handle Optional (Union with None)
            args = getattr(type_hint, '__args__', ())
            if len(args) == 2 and type(None) in args:
                # Optional type
                non_none_type = args[0] if args[1] is type(None) else args[1]
                schema = SchemaExtractor.extract_from_type(non_none_type)
                schema.nullable = True
                return schema

        elif origin is list:
            args = getattr(type_hint, '__args__', ())
            item_type = args[0] if args else str
            return Schema(
                type="array",
                items=SchemaExtractor.extract_from_type(item_type),
            )

        elif origin is dict:
            return Schema(type="object")

        # Try to extract from class
        if inspect.isclass(type_hint):
            # Try Pydantic first
            try:
                from pydantic import BaseModel
                if issubclass(type_hint, BaseModel):
                    return SchemaExtractor.extract_from_pydantic(type_hint)
            except ImportError:
                pass

            # Try DRF serializer
            try:
                from rest_framework import serializers
                if issubclass(type_hint, serializers.Serializer):
                    return SchemaExtractor.extract_from_serializer(type_hint)
            except ImportError:
                pass

            # Try Django model
            try:
                from django.db import models
                if issubclass(type_hint, models.Model):
                    return SchemaExtractor.extract_from_model(type_hint)
            except ImportError:
                pass

            # Try dataclass
            if is_dataclass(type_hint):
                return SchemaExtractor.extract_from_dataclass(type_hint)

        # Default to string
        return Schema(type="string")

    @staticmethod
    def extract_from_dataclass(dataclass_type: type) -> Schema:
        """Extract schema from dataclass

        Args:
            dataclass_type: Dataclass type

        Returns:
            Schema object
        """
        if not is_dataclass(dataclass_type):
            return Schema(type="object")

        properties = {}
        required = []

        # Get type hints
        hints = get_type_hints(dataclass_type)

        for field_name, field_type in hints.items():
            field_schema = SchemaExtractor.extract_from_type(field_type)
            properties[field_name] = field_schema

            # Check if field has no default (required)
            for field_info in dataclass_type.__dataclass_fields__.values():
                if field_info.name == field_name:
                    if field_info.default is dataclass_type.__dataclass_fields__[field_name].default_factory is None:
                        required.append(field_name)

        return Schema(
            type="object",
            properties=properties,
            required=required if required else None,
        )


def extract_schema(obj: Any) -> Schema:
    """Extract schema from any supported object type

    Args:
        obj: Object to extract schema from (serializer, model, type, etc.)

    Returns:
        Schema object
    """
    if obj is None:
        return Schema(type="object")

    # Try DRF serializer
    try:
        from rest_framework import serializers
        if inspect.isclass(obj) and issubclass(obj, serializers.Serializer):
            return SchemaExtractor.extract_from_serializer(obj)
    except ImportError:
        pass

    # Try Pydantic model
    try:
        from pydantic import BaseModel
        if inspect.isclass(obj) and issubclass(obj, BaseModel):
            return SchemaExtractor.extract_from_pydantic(obj)
    except ImportError:
        pass

    # Try Django model
    try:
        from django.db import models
        if inspect.isclass(obj) and issubclass(obj, models.Model):
            return SchemaExtractor.extract_from_model(obj)
    except ImportError:
        pass

    # Try dataclass
    if is_dataclass(obj):
        return SchemaExtractor.extract_from_dataclass(obj)

    # Try type hint
    return SchemaExtractor.extract_from_type(obj)



"""
ByteDocs Django - Configuration Management
Load and validate configuration from environment variables
"""

import os
from typing import Optional, List, Dict, Any
from .types import ByteDocsConfig, AuthConfig, UIConfig, AIConfig, AIFeatures, BaseURLOption


def load_dotenv_file() -> None:
    """Load .env file if it exists

    Searches for .env file in:
    1. Current working directory
    2. Parent directories (up to 3 levels)
    """
    try:
        from dotenv import load_dotenv, find_dotenv
        # find_dotenv() searches up the directory tree for .env
        dotenv_path = find_dotenv(usecwd=True)
        if dotenv_path:
            load_dotenv(dotenv_path=dotenv_path)
        else:
            # Fallback to current directory
            load_dotenv()
    except ImportError:
        # python-dotenv not installed, skip
        pass


def load_config_from_env() -> ByteDocsConfig:
    """Load configuration from environment variables

    Supports two naming conventions:
    - BYTEDOCS_* prefix (e.g., BYTEDOCS_TITLE)
    - Simple names (e.g., API_TITLE, API_VERSION)

    BYTEDOCS_* prefix takes priority if both are set.
    """
    # Load .env file first
    load_dotenv_file()

    config = ByteDocsConfig()

    # Basic config - support both BYTEDOCS_* and API_* prefixes
    config.title = os.getenv("BYTEDOCS_TITLE") or os.getenv("API_TITLE", "API Documentation")
    config.version = os.getenv("BYTEDOCS_VERSION") or os.getenv("API_VERSION", "1.0.0")
    config.description = os.getenv("BYTEDOCS_DESCRIPTION") or os.getenv("API_DESCRIPTION")
    config.base_url = os.getenv("BYTEDOCS_BASE_URL") or os.getenv("API_BASE_URL")
    config.docs_path = os.getenv("BYTEDOCS_DOCS_PATH") or os.getenv("DOCS_PATH", "/docs")
    config.auto_detect = os.getenv("BYTEDOCS_AUTO_DETECT", "true").lower() == "true"

    # Exclude paths
    exclude_paths = os.getenv("BYTEDOCS_EXCLUDE_PATHS", "")
    if exclude_paths:
        config.exclude_paths = [p.strip() for p in exclude_paths.split(",")]

    # Base URLs (multiple environments)
    # Support individual environment URLs
    base_urls_list = []
    production_url = os.getenv("BYTEDOCS_PRODUCTION_URL")
    local_url = os.getenv("BYTEDOCS_LOCAL_URL")

    if production_url:
        base_urls_list.append(BaseURLOption(name="Production", url=production_url))
    if local_url:
        base_urls_list.append(BaseURLOption(name="Local", url=local_url))

    # Also support old BYTEDOCS_BASE_URLS format for backward compatibility
    base_urls_str = os.getenv("BYTEDOCS_BASE_URLS")
    if base_urls_str:
        # Format: "Development:http://localhost:8000,Production:https://api.example.com"
        for url_pair in base_urls_str.split(","):
            if ":" in url_pair:
                parts = url_pair.split(":", 1)
                if len(parts) == 2:
                    name, url = parts
                    # Handle URLs with protocol
                    if url.startswith("//"):
                        url = "http:" + url
                    elif not url.startswith("http"):
                        # If there's more colons (like http://), reconstruct
                        remaining = url_pair.split(":", 2)
                        if len(remaining) >= 3:
                            name = remaining[0]
                            url = remaining[1] + ":" + remaining[2]
                    base_urls_list.append(BaseURLOption(name=name.strip(), url=url.strip()))

    if base_urls_list:
        config.base_urls = base_urls_list

    # Auth config
    auth_enabled = os.getenv("BYTEDOCS_AUTH_ENABLED", "false").lower() == "true"
    if auth_enabled:
        config.auth_config = AuthConfig(
            enabled=True,
            type=os.getenv("BYTEDOCS_AUTH_TYPE", "session"),
            username=os.getenv("BYTEDOCS_AUTH_USERNAME"),
            password=os.getenv("BYTEDOCS_AUTH_PASSWORD"),
            api_key=os.getenv("BYTEDOCS_AUTH_API_KEY"),
            api_key_header=os.getenv("BYTEDOCS_AUTH_API_KEY_HEADER", "X-API-Key"),
            realm=os.getenv("BYTEDOCS_AUTH_REALM"),
            session_expire=int(os.getenv("BYTEDOCS_AUTH_SESSION_EXPIRE", "60")),
            ip_ban_enabled=os.getenv("BYTEDOCS_AUTH_IP_BAN_ENABLED", "false").lower() == "true",
            ip_ban_max_attempts=int(os.getenv("BYTEDOCS_AUTH_IP_BAN_MAX_ATTEMPTS", "5")),
            ip_ban_duration=int(os.getenv("BYTEDOCS_AUTH_IP_BAN_DURATION", "30")),
        )

        # Admin whitelist IPs
        whitelist_ips = os.getenv("BYTEDOCS_AUTH_ADMIN_WHITELIST_IPS", "")
        if whitelist_ips:
            config.auth_config.admin_whitelist_ips = [ip.strip() for ip in whitelist_ips.split(",")]

    # UI config
    config.ui_config = UIConfig(
        theme=os.getenv("BYTEDOCS_UI_THEME", "green"),
        dark_mode=os.getenv("BYTEDOCS_UI_DARK_MODE", "false").lower() == "true",
        show_try_it=os.getenv("BYTEDOCS_UI_SHOW_TRY_IT", "true").lower() == "true",
        show_schemas=os.getenv("BYTEDOCS_UI_SHOW_SCHEMAS", "true").lower() == "true",
    )

    # AI config
    ai_enabled = os.getenv("BYTEDOCS_AI_ENABLED", "false").lower() == "true"
    if ai_enabled:
        # Parse max_tokens and max_completion_tokens
        max_tokens = None
        max_completion_tokens = None
        if os.getenv("BYTEDOCS_AI_MAX_TOKENS"):
            max_tokens = int(os.getenv("BYTEDOCS_AI_MAX_TOKENS"))
        if os.getenv("BYTEDOCS_AI_MAX_COMPLETION_TOKENS"):
            max_completion_tokens = int(os.getenv("BYTEDOCS_AI_MAX_COMPLETION_TOKENS"))

        config.ai_config = AIConfig(
            enabled=True,
            provider=os.getenv("BYTEDOCS_AI_PROVIDER", "openai"),
            api_key=os.getenv("BYTEDOCS_AI_API_KEY"),
            features=AIFeatures(
                chat_enabled=os.getenv("BYTEDOCS_AI_CHAT_ENABLED", "true").lower() == "true",
                doc_generation_enabled=os.getenv("BYTEDOCS_AI_DOC_GEN_ENABLED", "true").lower() == "true",
                model=os.getenv("BYTEDOCS_AI_MODEL"),
                max_tokens=max_tokens,
                max_completion_tokens=max_completion_tokens,
                temperature=float(os.getenv("BYTEDOCS_AI_TEMPERATURE", "0.7")),
            ),
        )

    return config


def validate_config(config: ByteDocsConfig) -> tuple[bool, List[str]]:
    """Validate configuration and return validation result"""
    errors = []

    # Validate docs path
    if not config.docs_path.startswith("/"):
        errors.append("docs_path must start with /")

    # Validate auth config
    if config.auth_config and config.auth_config.enabled:
        if config.auth_config.type == "basic":
            if not config.auth_config.username or not config.auth_config.password:
                errors.append("Basic auth requires username and password")
        elif config.auth_config.type == "api_key":
            if not config.auth_config.api_key:
                errors.append("API key auth requires api_key")
        elif config.auth_config.type == "bearer":
            if not config.auth_config.api_key:
                errors.append("Bearer auth requires api_key")
        elif config.auth_config.type == "session":
            if not config.auth_config.password:
                errors.append("Session auth requires password")

    # Validate AI config
    if config.ai_config and config.ai_config.enabled:
        if not config.ai_config.api_key:
            errors.append("AI features require api_key")

    return len(errors) == 0, errors


def merge_configs(env_config: ByteDocsConfig, user_config: Dict[str, Any]) -> ByteDocsConfig:
    """Merge environment config with user-provided config"""
    # Start with env config
    merged = env_config

    # Override with user config
    if "title" in user_config:
        merged.title = user_config["title"]
    if "version" in user_config:
        merged.version = user_config["version"]
    if "description" in user_config:
        merged.description = user_config["description"]
    if "base_url" in user_config:
        merged.base_url = user_config["base_url"]
    if "base_urls" in user_config:
        merged.base_urls = [
            BaseURLOption(**url) if isinstance(url, dict) else url
            for url in user_config["base_urls"]
        ]
    if "docs_path" in user_config:
        merged.docs_path = user_config["docs_path"]
    if "auto_detect" in user_config:
        merged.auto_detect = user_config["auto_detect"]
    if "exclude_paths" in user_config:
        merged.exclude_paths = user_config["exclude_paths"]

    # Auth config
    if "auth_config" in user_config:
        auth = user_config["auth_config"]
        if isinstance(auth, dict):
            merged.auth_config = AuthConfig(**auth)
        else:
            merged.auth_config = auth

    # UI config
    if "ui_config" in user_config:
        ui = user_config["ui_config"]
        if isinstance(ui, dict):
            merged.ui_config = UIConfig(**ui)
        else:
            merged.ui_config = ui

    # AI config
    if "ai_config" in user_config:
        ai = user_config["ai_config"]
        if isinstance(ai, dict):
            features = ai.get("features", {})
            if isinstance(features, dict):
                ai["features"] = AIFeatures(**features)
            merged.ai_config = AIConfig(**ai)
        else:
            merged.ai_config = ai

    return merged



"""
ByteDocs Django - UI Handlers
Create view functions for documentation endpoints
"""

import json
import yaml
from typing import Any, Dict, Optional
from django.http import HttpRequest, HttpResponse, JsonResponse
from .template_loader import load_template, render_template


def create_docs_ui_handler(bytedocs_instance):
    """Create documentation UI handler

    Args:
        bytedocs_instance: ByteDocs instance with documentation data

    Returns:
        Django view function
    """
    def docs_ui(request: HttpRequest) -> HttpResponse:
        """Serve documentation UI"""
        # Auto-detect routes if needed
        if bytedocs_instance.config.auto_detect and not bytedocs_instance._detected:
            bytedocs_instance.detect_routes()

        # Get documentation data
        doc_data = bytedocs_instance.get_documentation_data()

        # Load template
        template = load_template("template.html")

        # Prepare data for template
        template_data = {
            "title": bytedocs_instance.config.title,
            "description": bytedocs_instance.config.description or "",
            "api_data": json.dumps(doc_data, default=str),
            "config_data": json.dumps({
                "theme": bytedocs_instance.config.ui_config.theme,
                "darkMode": bytedocs_instance.config.ui_config.dark_mode,
                "showTryIt": bytedocs_instance.config.ui_config.show_try_it,
                "showSchemas": bytedocs_instance.config.ui_config.show_schemas,
                "aiEnabled": bytedocs_instance.config.ai_config.enabled if bytedocs_instance.config.ai_config else False,
                "chatEnabled": (
                    bytedocs_instance.config.ai_config.features.chat_enabled
                    if bytedocs_instance.config.ai_config and bytedocs_instance.config.ai_config.enabled
                    else False
                ),
            }, default=str),
        }

        # Render template
        html = render_template(template, template_data)

        return HttpResponse(html, content_type="text/html")

    return docs_ui


def create_api_data_handler(bytedocs_instance):
    """Create API data JSON handler

    Args:
        bytedocs_instance: ByteDocs instance with documentation data

    Returns:
        Django view function
    """
    def api_data(request: HttpRequest) -> JsonResponse:
        """Serve API documentation data as JSON"""
        # Auto-detect routes if needed
        if bytedocs_instance.config.auto_detect and not bytedocs_instance._detected:
            bytedocs_instance.detect_routes()

        # Get documentation data
        doc_data = bytedocs_instance.get_documentation_data()

        return JsonResponse(doc_data, safe=False)

    return api_data


def create_openapi_json_handler(bytedocs_instance):
    """Create OpenAPI JSON handler

    Args:
        bytedocs_instance: ByteDocs instance with documentation data

    Returns:
        Django view function
    """
    def openapi_json(request: HttpRequest) -> JsonResponse:
        """Serve OpenAPI specification as JSON"""
        # Auto-detect routes if needed
        if bytedocs_instance.config.auto_detect and not bytedocs_instance._detected:
            bytedocs_instance.detect_routes()

        # Get OpenAPI spec
        openapi_spec = bytedocs_instance.get_openapi_spec()

        return JsonResponse(openapi_spec, safe=False)

    return openapi_json


def create_openapi_yaml_handler(bytedocs_instance):
    """Create OpenAPI YAML handler

    Args:
        bytedocs_instance: ByteDocs instance with documentation data

    Returns:
        Django view function
    """
    def openapi_yaml(request: HttpRequest) -> HttpResponse:
        """Serve OpenAPI specification as YAML"""
        # Auto-detect routes if needed
        if bytedocs_instance.config.auto_detect and not bytedocs_instance._detected:
            bytedocs_instance.detect_routes()

        # Get OpenAPI spec
        openapi_spec = bytedocs_instance.get_openapi_spec()

        # Convert to YAML
        yaml_content = yaml.dump(openapi_spec, sort_keys=False, allow_unicode=True)

        return HttpResponse(yaml_content, content_type="application/x-yaml")

    return openapi_yaml


def create_chat_handler(bytedocs_instance):
    """Create AI chat handler

    Args:
        bytedocs_instance: ByteDocs instance with documentation data

    Returns:
        Django view function
    """
    def chat(request: HttpRequest) -> JsonResponse:
        """Handle AI chat requests"""
        # Check if AI is enabled
        if not bytedocs_instance.config.ai_config or not bytedocs_instance.config.ai_config.enabled:
            return JsonResponse(
                {"error": "AI features are not enabled"},
                status=400
            )

        # Check if chat is enabled
        if not bytedocs_instance.config.ai_config.features.chat_enabled:
            return JsonResponse(
                {"error": "Chat feature is not enabled"},
                status=400
            )

        # Only accept POST requests
        if request.method != "POST":
            return JsonResponse(
                {"error": "Method not allowed"},
                status=405
            )

        try:
            # Parse request body
            data = json.loads(request.body)
            message = data.get("message")
            endpoint_id = data.get("endpoint_id")

            if not message:
                return JsonResponse(
                    {"error": "Message is required"},
                    status=400
                )

            # Get AI response
            response = bytedocs_instance.handle_chat(message, endpoint_id)

            return JsonResponse(response)

        except json.JSONDecodeError:
            return JsonResponse(
                {"error": "Invalid JSON"},
                status=400
            )
        except Exception as e:
            return JsonResponse(
                {"error": str(e)},
                status=500
            )

    return chat



"""
ByteDocs Django - URL Configuration
URL patterns for documentation endpoints
"""

from django.urls import path
from . import views

app_name = 'bytedocs'

urlpatterns = [
    path('', views.docs_ui, name='docs_ui'),
    path('api-data.json', views.api_data, name='api_data'),
    path('openapi.json', views.openapi_json, name='openapi_json'),
    path('openapi.yaml', views.openapi_yaml, name='openapi_yaml'),
    path('chat', views.ai_chat, name='ai_chat'),
]
